part of '../fl_camera.dart';

typedef FlCameraCreateCallback = void Function(FlCameraController controller);

class FlCamera extends StatefulWidget {
  const FlCamera(
      {Key? key,
      this.overlay,
      this.uninitialized,
      this.onFlashChanged,
      this.onZoomChanged,
      this.updateReset = false,
      this.camera,
      this.resolution = CameraResolution.high,
      this.fit = BoxFit.fitWidth,
      this.onCreateView,
      this.notPreviewed})
      : super(key: key);

  /// 显示在预览框上面
  /// Display above preview box
  final Widget? overlay;

  /// 相机在未初始化时显示的UI
  /// The UI displayed when the camera is not initialized
  final Widget? uninitialized;

  /// 停止预览时显示的UI
  /// The UI displayed when the camera is not previewed
  final Widget? notPreviewed;

  /// 闪光灯变化
  /// Flash change
  final ValueChanged<FlashState>? onFlashChanged;

  /// 缩放变化
  /// zoom ratio
  final ValueChanged<CameraZoomState>? onZoomChanged;

  /// 更新组件时是否重置相机
  /// Reset camera when updating components
  final bool updateReset;

  /// 需要预览的相机
  /// Camera ID to preview
  final CameraInfo? camera;

  /// 预览相机支持的分辨率
  /// Preview the resolution supported by the camera
  final CameraResolution resolution;

  /// How a camera box should be inscribed into another box.
  final BoxFit fit;

  /// get Controller
  final FlCameraCreateCallback? onCreateView;

  @override
  _FlCameraStateWidget createState() => _FlCameraStateWidget();
}

class _FlCameraStateWidget extends FlCameraComposeState<FlCamera> {
  @override
  void initState() {
    controller = FlCameraController();
    super.initState();
    uninitialized = widget.uninitialized;
    notPreviewed = widget.notPreviewed;
    if (widget.onCreateView != null) {
      widget.onCreateView!(controller as FlCameraController);
    }
    WidgetsBinding.instance!.addPostFrameCallback((_) => initialize());
  }

  Future<void> initialize() async {
    boxFit = widget.fit;
    var camera = widget.camera;
    if (camera == null) {
      final List<CameraInfo>? cameras = await controller.availableCameras();
      if (cameras == null) return;
      for (final CameraInfo cameraInfo in cameras) {
        if (cameraInfo.lensFacing == CameraLensFacing.back) {
          camera = cameraInfo;
          break;
        }
      }
    }
    if (camera == null) return;
    final data = await controller.initialize();
    if (data) {
      initializeListen();
      final options = await controller.startPreview(camera.name);
      if (options != null && mounted) {
        setState(() {});
      }
    }
  }

  void initializeListen() {
    if (widget.onZoomChanged != null) {
      controller.onZoomChanged = widget.onZoomChanged;
    }
    if (widget.onFlashChanged != null) {
      controller.onFlashChanged = widget.onFlashChanged;
    }
  }

  @override
  void didUpdateWidget(covariant FlCamera oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.overlay != widget.overlay ||
        oldWidget.onFlashChanged != widget.onFlashChanged ||
        oldWidget.onZoomChanged != widget.onZoomChanged ||
        oldWidget.notPreviewed != widget.notPreviewed ||
        oldWidget.uninitialized != widget.uninitialized) {
      if (widget.updateReset) {
        controller.dispose().then((value) {
          initialize();
        });
      }
    }
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      initialize();
    } else {
      super.didChangeAppLifecycleState(state);
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget camera = super.build(context);
    if (widget.overlay != null) {
      camera = Stack(children: <Widget>[
        camera,
        SizedBox.expand(child: widget.overlay),
      ]);
    }
    return camera;
  }

  @override
  void dispose() {
    super.dispose();
    controller.dispose();
  }
}

abstract class FlCameraComposeState<T extends StatefulWidget>
    extends FlCameraState<T> with WidgetsBindingObserver {
  /// 相机在未初始化时显示的UI
  /// The UI displayed when the camera is not initialized
  Widget? uninitialized;

  /// 是否全屏 全屏会拉伸相机画面
  /// fullScreen
  bool fullScreen = false;

  /// camera fit
  BoxFit? boxFit;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state != AppLifecycleState.resumed) controller.dispose();
  }

  @override
  Widget build(BuildContext context) {
    boxFit ??= fullScreen ? BoxFit.fitHeight : BoxFit.fitWidth;
    Widget current = super.build(context);
    final FlCameraOptions? cameraOptions = controller.cameraOptions?.value;
    if (cameraOptions != null && cameraOptions.textureId != null) {
      current = ColoredBox(
          color: Colors.black,
          child: FittedBox(
              fit: boxFit!,
              child: SizedBox.fromSize(
                  size: Size(cameraOptions.width!, cameraOptions.height!),
                  child: current)));
    } else {
      current = uninitialized ??
          Container(
              color: Colors.black,
              alignment: Alignment.center,
              child: const Text('Camera not initialized',
                  style: TextStyle(color: Colors.white)));
    }
    return SizedBox.expand(child: current);
  }
}

abstract class FlCameraState<T extends StatefulWidget> extends State<T>
    with WidgetsBindingObserver {
  late CameraController controller;

  /// 相机在暂停预览时显示的UI
  /// The UI displayed when the camera is not previewed
  Widget? notPreviewed;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addObserver(this);
    WidgetsBinding.instance!.addPostFrameCallback((Duration duration) async {
      await SystemChrome.setPreferredOrientations(
          <DeviceOrientation>[DeviceOrientation.portraitUp]);
      SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle.light
          .copyWith(statusBarColor: Colors.transparent));
    });
  }

  @override
  Widget build(BuildContext context) {
    final cameraOptions = controller.cameraOptions;
    if (cameraOptions != null) {
      return ValueListenableBuilder(
          valueListenable: cameraOptions,
          builder: (_, FlCameraOptions? options, __) {
            if (options != null && options.textureId != null) {
              return Texture(textureId: options.textureId!);
            }
            return notPreviewed ?? const SizedBox();
          });
    }
    return const SizedBox();
  }

  @override
  void dispose() {
    super.dispose();
    WidgetsBinding.instance!.removeObserver(this);
  }
}
